/*
 * Copyright (c) 2017-2019, Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 */

#ifndef __CORE_TEST_DEFS_H__
#define __CORE_TEST_DEFS_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <inttypes.h>
#include <limits.h>
#include "tfm_api.h"

/* These definitions are used in symbols, only digits are permitted */
#define CORE_TEST_ID_NS_THREAD            1001
#define CORE_TEST_ID_CHECK_INIT           1003
#define CORE_TEST_ID_RECURSION            1004
#define CORE_TEST_ID_MEMORY_PERMISSIONS   1005
#define CORE_TEST_ID_MPU_ACCESS           1006
#define CORE_TEST_ID_BUFFER_CHECK         1007
#define CORE_TEST_ID_SS_TO_SS             1008
#define CORE_TEST_ID_SHARE_REDIRECTION    1009
#define CORE_TEST_ID_SS_TO_SS_BUFFER      1010
#define CORE_TEST_ID_PERIPHERAL_ACCESS    1012
#define CORE_TEST_ID_GET_CALLER_CLIENT_ID 1013
#define CORE_TEST_ID_SPM_REQUEST          1014
#define CORE_TEST_ID_IOVEC_SANITIZATION   1015
#define CORE_TEST_ID_OUTVEC_WRITE         1016
#define CORE_TEST_ID_BLOCK                2001

#define SPM_CORE_TEST_INIT_SUCCESS_SID                 0x00002100
#define SPM_CORE_TEST_DIRECT_RECURSION_SID             0x00002101
#define SPM_CORE_TEST_MPU_ACCESS_SID                   0x00002102
#define SPM_CORE_TEST_MEMORY_PERMISSIONS_SID           0x00002103
#define SPM_CORE_TEST_SHARE_REDIRECTION_SID            0x00002104
#define SPM_CORE_TEST_SS_TO_SS_SID                     0x00002105
#define SPM_CORE_TEST_SS_TO_SS_BUFFER_SID              0x00002106
#define SPM_CORE_TEST_OUTVEC_WRITE_SID                 0x00002107
#define SPM_CORE_TEST_PERIPHERAL_ACCESS_SID            0x00002108
#define SPM_CORE_TEST_GET_CALLER_CLIENT_ID_SID         0x00002109
#define SPM_CORE_TEST_SPM_REQUEST_SID                  0x0000210A
#define SPM_CORE_TEST_BLOCK_SID                        0x0000210B
#define SPM_CORE_TEST_NS_THREAD_SID                    0x0000210C
#define SPM_CORE_TEST_INIT_SUCCESS_MIN_VER                 0x0001
#define SPM_CORE_TEST_DIRECT_RECURSION_MIN_VER             0x0001
#define SPM_CORE_TEST_MPU_ACCESS_MIN_VER                   0x0001
#define SPM_CORE_TEST_MEMORY_PERMISSIONS_MIN_VER           0x0001
#define SPM_CORE_TEST_SHARE_REDIRECTION_MIN_VER            0x0001
#define SPM_CORE_TEST_SS_TO_SS_MIN_VER                     0x0001
#define SPM_CORE_TEST_SS_TO_SS_BUFFER_MIN_VER              0x0001
#define SPM_CORE_TEST_OUTVEC_WRITE_MIN_VER                 0x0001
#define SPM_CORE_TEST_PERIPHERAL_ACCESS_MIN_VER            0x0001
#define SPM_CORE_TEST_GET_CALLER_CLIENT_ID_MIN_VER         0x0001
#define SPM_CORE_TEST_SPM_REQUEST_MIN_VER                  0x0001
#define SPM_CORE_TEST_BLOCK_MIN_VER                        0x0001
#define SPM_CORE_TEST_NS_THREAD_MIN_VER                    0x0001

#define SPM_CORE_TEST_2_SLAVE_SERVICE_SID              0x00002200
#define SPM_CORE_TEST_2_CHECK_CALLER_CLIENT_ID_SID     0x00002201
#define SPM_CORE_TEST_2_GET_EVERY_SECOND_BYTE_SID      0x00002202
#define SPM_CORE_TEST_2_INVERT_SID                     0x00002203
#define SPM_CORE_TEST_2_SLAVE_SERVICE_MIN_VER              0x0001
#define SPM_CORE_TEST_2_CHECK_CALLER_CLIENT_ID_MIN_VER     0x0001
#define SPM_CORE_TEST_2_GET_EVERY_SECOND_BYTE_MIN_VER      0x0001
#define SPM_CORE_TEST_2_INVERT_MIN_VER                     0x0001

/* Use lower 16 bits in return value for error code, upper 16 for line number
 * in test service
 */
#define CORE_TEST_RETURN_ERROR(x) return (((__LINE__) << 16) | x)
#define CORE_TEST_ERROR_GET_EXTRA(x) (x >> 16)
#define CORE_TEST_ERROR_GET_CODE(x) (x & 0xFFFF)

enum core_test_errno_t {
    CORE_TEST_ERRNO_SP_NOT_INITED              = -12,
    CORE_TEST_ERRNO_UNEXPECTED_CORE_BEHAVIOUR  = -11,
    CORE_TEST_ERRNO_SP_RECURSION_NOT_REJECTED  = -10,
    CORE_TEST_ERRNO_INVALID_BUFFER             = -9,
    CORE_TEST_ERRNO_SLAVE_SP_CALL_FAILURE      = -8,
    CORE_TEST_ERRNO_SLAVE_SP_BUFFER_FAILURE    = -7,
    CORE_TEST_ERRNO_FIRST_CALL_FAILED          = -6,
    CORE_TEST_ERRNO_SECOND_CALL_FAILED         = -5,
    CORE_TEST_ERRNO_PERIPHERAL_ACCESS_FAILED   = -4,
    CORE_TEST_ERRNO_TEST_FAULT                 = -3,
    CORE_TEST_ERRNO_INVALID_TEST_ID            = -2,
    CORE_TEST_ERRNO_INVALID_PARAMETER          = -1,

    CORE_TEST_ERRNO_SUCCESS                    =  0,

    CORE_TEST_ERRNO_SUCCESS_2                  =  1,

    /* Following entry is only to ensure the error code of int size */
    CORE_TEST_ERRNO_FORCE_INT_SIZE = INT_MAX
};

#ifdef __cplusplus
}
#endif

#endif /* __CORE_TEST_DEFS_H__ */
